Re: [PATCH v3 3/5] config: make parsing stack struct independent from actual data source

2013-05-09 Thread Jeff King
On Thu, May 09, 2013 at 06:19:32PM +0200, Heiko Voigt wrote:

> diff --git a/config.c b/config.c
> index 046642b..2390458 100644
> --- a/config.c
> +++ b/config.c
> @@ -10,20 +10,41 @@
>  #include "strbuf.h"
>  #include "quote.h"
>  
> -typedef struct config_file {
> - struct config_file *prev;
> - FILE *f;
> +struct config_source {
> + struct config_source *prev;
> + union {
> + FILE *file;
> + };

Anonymous unions like this are a C11-ism (I don't know when gcc learned
about them, but I am sure that many of the older compilers we support
would not be happy). You have to do:

  union {
  FILE *file;
  } u;

and access the file as "cf->u.file".

-Peff
--
To unsubscribe from this list: send the line "unsubscribe git" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html


[PATCH v3 3/5] config: make parsing stack struct independent from actual data source

2013-05-09 Thread Heiko Voigt
To simplify adding other sources we extract all functions needed for
parsing into a list of callbacks. We implement those callbacks for the
current file parsing. A new source can implement its own set of callbacks.

Instead of storing the concrete FILE pointer for parsing we store a void
pointer. A new source can use this to store its custom data.

Signed-off-by: Heiko Voigt 
---
 config.c | 66 ++--
 1 file changed, 44 insertions(+), 22 deletions(-)

diff --git a/config.c b/config.c
index 046642b..2390458 100644
--- a/config.c
+++ b/config.c
@@ -10,20 +10,41 @@
 #include "strbuf.h"
 #include "quote.h"
 
-typedef struct config_file {
-   struct config_file *prev;
-   FILE *f;
+struct config_source {
+   struct config_source *prev;
+   union {
+   FILE *file;
+   };
const char *name;
int linenr;
int eof;
struct strbuf value;
struct strbuf var;
-} config_file;
 
-static config_file *cf;
+   int (*fgetc)(struct config_source *c);
+   int (*ungetc)(int c, struct config_source *conf);
+   long (*ftell)(struct config_source *c);
+};
+
+static struct config_source *cf;
 
 static int zlib_compression_seen;
 
+static int config_file_fgetc(struct config_source *conf)
+{
+   return fgetc(conf->file);
+}
+
+static int config_file_ungetc(int c, struct config_source *conf)
+{
+   return ungetc(c, conf->file);
+}
+
+static long config_file_ftell(struct config_source *conf)
+{
+   return ftell(conf->file);
+}
+
 #define MAX_INCLUDE_DEPTH 10
 static const char include_depth_advice[] =
 "exceeded maximum include depth (%d) while including\n"
@@ -168,15 +189,13 @@ int git_config_from_parameters(config_fn_t fn, void *data)
 
 static int get_next_char(void)
 {
-   int c;
-   FILE *f = cf->f;
+   int c = cf->fgetc(cf);
 
-   c = fgetc(f);
if (c == '\r') {
/* DOS like systems */
-   c = fgetc(f);
+   c = cf->fgetc(cf);
if (c != '\n') {
-   ungetc(c, f);
+   cf->ungetc(c, cf);
c = '\r';
}
}
@@ -336,7 +355,7 @@ static int get_base_var(struct strbuf *name)
}
 }
 
-static int git_parse_file(config_fn_t fn, void *data)
+static int git_parse_source(config_fn_t fn, void *data)
 {
int comment = 0;
int baselen = 0;
@@ -894,10 +913,11 @@ int git_default_config(const char *var, const char 
*value, void *dummy)
 }
 
 /*
  * The fields f and name of top need to be initialized before calling
  * this function.
  */
-static int do_config_from(struct config_file *top, config_fn_t fn, void *data)
+static int do_config_from_source(struct config_source *top, config_fn_t fn, 
void *data)
 {
int ret;
 
@@ -909,7 +929,7 @@ static int do_config_from(struct config_file *top, 
config_fn_t fn, void *data)
strbuf_init(&top->var, 1024);
cf = top;
 
-   ret = git_parse_file(fn, data);
+   ret = git_parse_source(fn, data);
 
/* pop config-file parsing state stack */
strbuf_release(&top->value);
@@ -926,12 +946,15 @@ int git_config_from_file(config_fn_t fn, const char 
*filename, void *data)
 
ret = -1;
if (f) {
-   config_file top;
+   struct config_source top;
 
-   top.f = f;
+   top.file = f;
top.name = filename;
+   top.fgetc = config_file_fgetc;
+   top.ungetc = config_file_ungetc;
+   top.ftell = config_file_ftell;
 
-   ret = do_config_from(&top, fn, data);
+   ret = do_config_from_source(&top, fn, data);
 
fclose(f);
}
@@ -1064,7 +1087,6 @@ static int store_aux(const char *key, const char *value, 
void *cb)
 {
const char *ep;
size_t section_len;
-   FILE *f = cf->f;
 
switch (store.state) {
case KEY_SEEN:
@@ -1076,7 +1098,7 @@ static int store_aux(const char *key, const char *value, 
void *cb)
return 1;
}
 
-   store.offset[store.seen] = ftell(f);
+   store.offset[store.seen] = cf->ftell(cf);
store.seen++;
}
break;
@@ -1103,19 +1125,19 @@ static int store_aux(const char *key, const char 
*value, void *cb)
 * Do not increment matches: this is no match, but we
 * just made sure we are in the desired section.
 */
-   store.offset[store.seen] = ftell(f);
+   store.offset[store.seen] = cf->ftell(cf);
@@ -894,10 +913,11 @@ int git_default_config(const char *var, const char 
 
 /*
- * The fields f and name of top need to be initialized before calling
+ * All source specific fields in the union, name and the callbacks
+ * fgetc, ungetc, ftell of top need to be initialized before calling
  * this function.
  */
/* fallthru */
case SECTION_END_SEEN:
case START:
if (matches(key, val